package cokr.xit.fims.levy.dao;

import java.util.List;
import java.util.Map;

import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import cokr.xit.foundation.component.AbstractMapper;
import cokr.xit.foundation.data.DataObject;

import cokr.xit.fims.levy.Rduct;
import cokr.xit.fims.crdn.Crdn;
import cokr.xit.fims.levy.Levy;
import cokr.xit.fims.levy.LevyQuery;

/** 과태료 감경 대장 정보 DAO
 *
 * <p>상세 설명:
 *
 * <pre>
 * ============ 변경 이력 ============
 * 2023-08-30	JoJH 최초 작성
 * ================================
 * </pre>
 */
@Mapper("rductMapper")
public interface RductMapper extends AbstractMapper {

	/**지정한 조건에 따라 과태료 감경 대장 목록을 조회하여 반환한다.<br />
	 * @param req 과태료 감경 대장 조회 조건
	 * @return 과태료 감경 대장 목록
	 */
	List<DataObject> selectRductList(LevyQuery req);

	/**지정한 조건에 따라 과태료 감경 대장 객체들을 반환한다.
	 * @param req 과태료 감경 대장 조회 조건
	 * @return 과태료 감경 대장 객체 목록
	 */
	List<DataObject> selectRducts(LevyQuery req);

	/**지정한 조건에 따라 과태료 감경 대장 객체의 마지막 1개를 반환한다.
	 * @param req 과태료 감경 대장 조회 조건
	 * @return 과태료 감경 대장 객체
	 */
	default DataObject selectRductInfo(LevyQuery req) {
		List<DataObject> rducts = selectRducts(req);

		return !rducts.isEmpty() ? rducts.get(0) : null;
	}

	/**과태료 감경 대장 정보를 등록한다.
	 * @param params 파라미터
	 * <ul><li>"rduct" - 과태료 감경 대장</li>
	 *     <li>"currentUser" - 현재 접속한 사용자</li>
	 * </ul>
	 * @return 저장된 정보수
	 */
	int insertRduct(Map<String, Object> params);

	/**과태료 감경 대장 정보를 등록한다.
	 * @param rduct 과태료 감경 대장
	 * @return 저장 여부
	 * <ul><li>저장됐으면 true</li>
	 *     <li>그렇지 않으면 false</li>
	 * </ul>
	 */
	default boolean insert(Rduct rduct) {
		return rduct != null && insertRduct(params().set("rduct", rduct)) == 1;
	}

	/**과태료 감경 대장 정보를 수정한다.
	 * @param params 파라미터
	 * <ul><li>"rduct" - 과태료 감경 대장</li>
	 *     <li>"currentUser" - 현재 접속한 사용자</li>
	 * </ul>
	 * @return 저장된 정보수
	 */
	int updateRduct(Map<String, Object> params);

	/**과태료 감경 대장 정보를 수정한다.
	 * @param rduct 과태료 감경 대장
	 * @return 저장 여부
	 * <ul><li>저장됐으면 true</li>
	 *     <li>그렇지 않으면 false</li>
	 * </ul>
	 */
	default boolean update(Rduct rduct) {
		return rduct != null && updateRduct(params().set("rduct", rduct)) == 1;
	}

	/**지정한 과태료 감경 대장을 삭제한다.
	 * @param params 파라미터
	 * <ul><li>"rduct" - 과태료 감경 대장</li>
	 *     <li>"currentUser" - 현재 접속한 사용자</li>
	 * </ul>
	 * @return 저장된 정보수
	 */
	int deleteRduct(Map<String, ?> params);

	/**과태료 감경 대장 정보를 삭제한다.
	 * @param rduct 과태료 감경 대장
	 * @return 저장 여부
	 * <ul><li>저장됐으면 true</li>
	 *     <li>그렇지 않으면 false</li>
	 * </ul>
	 */
	default boolean delete(Rduct rduct) {
		return rduct != null && deleteRduct(params().set("rduct", rduct)) == 1;
	}

	/**지정한 단속 ID의 단속, 부과제외 정보를 반환한다.
	 * @param crdnId 단속 ID
	 * @return 단속, 부과제외 정보
	 */
	DataObject selectCrdn(LevyQuery req);

	default DataObject selectCrdnInfo(String crdnId) {
		DataObject rductInfo = selectCrdn(new LevyQuery().setCrdnId(crdnId));

		return rductInfo;
	}

	/**단속(TB_CRDN) 대장의 금액을 수정한다.
	 * @param crdn 단속 대장
	 * @return 저장된 정보수
	 */
	int updateCrdnRduct(Map<String, Object> params);

	default boolean updateCrdn(Crdn crdn) {
		return crdn != null && updateCrdnRduct(params().set("crdn", crdn)) == 1;
	}

	/**부과(TB_LEVY) 대장의 금액을 수정한다.
	 * @param levy 부과 대장
	 * @return 저장된 정보수
	 */
	int updateLevyRduct(Map<String, Object> params);

	default boolean updateLevy(Levy levy) {
		return levy != null && updateLevyRduct(params().set("levy", levy)) == 1;
	}

}
